@@ -6,8 +6,9 @@
  * This program is open source, licensed under the PostgreSQL license.
  * For license terms, see the LICENSE file.
  *
- * Copyright (C) 2021-2023: MigOps Inc
- * Copyright (C) 2023: Gilles Darold
+ * Copyright (c) 2021-2023: MigOps Inc
+ * Copyright (c) 2023: Gilles Darold
+ * Copyright (c) 2024: HexaCluster Corp
  *
  *-------------------------------------------------------------------------
  */
@@ -227,6 +228,7 @@ static bool password_contain_username = true;
 static bool password_ignore_case = false;
 static int password_valid_until = 0;
 static int password_valid_max = 0;
+static int auth_delay_milliseconds = 0;
 
 #if PG_VERSION_NUM >= 120000
 /*
@@ -1314,8 +1316,21 @@ _PG_init(void)
 				gettext_noop("comma separated list of username to exclude from password policy check"), NULL,
 				&username_whitelist, "", PGC_SUSET, 0, check_whitelist, NULL, NULL);
 
+	DefineCustomIntVariable("credcheck.auth_delay_ms",
+				"Milliseconds to delay before reporting authentication failure",
+				NULL,
+				&auth_delay_milliseconds,
+				0,
+				0, INT_MAX / 1000,
+				PGC_SIGHUP,
+				GUC_UNIT_MS,
+				NULL,
+				NULL,
+				NULL);
 
 #if PG_VERSION_NUM < 150000
+	EmitWarningsOnPlaceholders("credcheck");
+
         /*
          * Request additional shared resources.  (These are no-ops if we're not in
          * the postmaster process.)  We'll allocate or attach to the shared
@@ -1325,6 +1340,9 @@ _PG_init(void)
         RequestNamedLWLockTranche(PGPH_TRANCHE_NAME, 1);
         RequestAddinShmemSpace(pgaf_memsize());
         RequestNamedLWLockTranche(PGAF_TRANCHE_NAME, 1);
+#else
+	MarkGUCPrefixReserved("credcheck");
+
 #endif
 
 	/* Install hooks */
@@ -1879,7 +1897,16 @@ flush_password_history(void)
                         goto error;
                 }
         }
+	/*
+	 * Fill the file until a size divisible by page size 8192
+	 * to fix a complain of pgBackRest backup: file size X is
+	 * not divisible by page size 8192
+	 */
+	fseek(file, 0, SEEK_END);
+	while ((ftell(file) % BLCKSZ) != 0)
+		putc(0, file);
 
+	/* close the file */
         if (FreeFile(file))
         {
                 file = NULL;
@@ -2154,6 +2181,12 @@ fix_log(ErrorData *edata)
 static void
 credcheck_max_auth_failure(Port *port, int status)
 {
+
+	/* Inject a short delay if authentication failed. */
+	if (status != STATUS_OK)
+		pg_usleep(1000L * auth_delay_milliseconds);
+
+	/* check for max auth failure */
 	if (fail_max > 0 && status != STATUS_EOF)
 	{
 		Oid userOid =  get_role_oid(port->user_name, true);
@@ -2162,7 +2195,7 @@ credcheck_max_auth_failure(Port *port, int status)
 		{
 			float fail_num = get_auth_failure(port->user_name, userOid, status);
 
-			/* register the auth failure if the we not reach allowed max failure */
+			/* register the auth failure if we not reach allowed max failure */
 			if (status == STATUS_ERROR && fail_num <= fail_max)
 				fail_num = save_auth_failure(port->user_name, userOid);
 
